using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using RazorPagesDownload.MyCode;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Threading.Tasks;

//====================================================
//       :
// https://www.interestprograms.ru
//     
//====================================================


namespace RazorPagesDownload.Pages
{
    public class PrivateFilesModel : PageModel
    {
        //      .
        public List<FileInfo> ListFiles { get; set; }



        public PrivateFilesModel()
        {
            //     .
            var di = new DirectoryInfo(MyConstants.PrivateDownloadDirectory);

            //var di = new DirectoryInfo(MyConstants.CustomDownloadDirectory);

            ListFiles = di.GetFiles().ToList();
        }



        public async Task<IActionResult> OnPostAsync(string file, string mode)
        {

            FileInfo fi = ListFiles.FirstOrDefault(p => p.Name == file);

            //      .
            fi.Refresh();

            if (fi.Exists == true)
            {

                return mode switch
                {
                    "memory" => await MemorySend(fi),
                    "stream" => await StreamSend(fi),
                    "bytes" => await BytesSend(fi),
                    _ => new EmptyResult(),
                };
            }
            else
            {
                //    .
            }


            return new EmptyResult();

        }



        #region    

        private async Task<IActionResult> BytesSend(FileInfo fi)
        {
            var bytes = await System.IO.File.ReadAllBytesAsync(fi.FullName);

            var contentType = MyUtility.ContentTypes(fi.Extension);


            // 1 
            return File(bytes, contentType, fi.Name);

            // 2 
            //return new FileContentResult(bytes, file_type)
            //{
            //    FileDownloadName = fi.Name
            //};
        }



        private async Task<IActionResult> StreamSend(FileInfo fi)
        {
            return await Task.Run(() =>
            {

                //  .
                var stream = fi.OpenRead(); //System.IO.File.OpenRead(path);

                var contentType = MyUtility.ContentTypes(fi.Extension);


                // 1 
                return File(stream, contentType, fi.Name);

                // 2 
                //return new FileStreamResult(stream, contentType)
                //{
                //    FileDownloadName = fi.Name
                //};
            });
        }



        private async Task<IActionResult> MemorySend(FileInfo fi)
        {
            var buffer = new MemoryStream();
            using var stream = fi.OpenRead();
            await stream.CopyToAsync(buffer);
            buffer.Position = 0;

            var contentType = MyUtility.ContentTypes(fi.Extension);

            // 1 
            //return File(buffer, contentType, fi.Name);

            // 2 
            return new FileStreamResult(buffer, contentType)
            {
                FileDownloadName = fi.Name
            };
        }

        #endregion

    }
}
